<?php

class Request
{
    var $name;
    var $attributes;

    function Request($name)
    {
        $this->name = $name;
        $this->attributes = array();
    }
    function setAttribute($attribute, $value)
    {
        $this->attributes[$attribute] = $value;
    }
    function getAttribute($attribute)
    {
        return $this->attributes[$attribute];
    }
    function setAttributeByRef($attribute, &$value)
    {
        $this->attributes[$attribute] = &$value;
    }
    function &getAttributeByRef($attribute)
    {
        return $this->attributes[$attribute];
    }
    function &session($reset = false)
    {
        $name = '__' . $this->name . '_session';
        if (!isset($_SESSION[$name]) || $reset) {
            $_SESSION[$name] = array();
        }
        return $_SESSION[$name];
    }
    function &getInstance($name)
    {
        static $request;
        if (!isset($request[$name])) {
            $request[$name] = new Request($name);
        }
        return $request[$name];
    }
}

class Controller
{
    var $name;
    var $dir;
    var $request;
    var $action;
    var $renderer;


    function Controller($name)
    {
        $this->name = $name;
        $this->request     =& Request::getInstance($name);
        $this->renderer    =& Renderer::getInstance();
        $this->renderer->setdir(dirname(__FILE__).DIRECTORY_SEPARATOR.'views'.DIRECTORY_SEPARATOR.'templates');
    }

    function forward($actionname)
    {
        if (!$actionname) return $this->view('index');

        $actionfile = dirname(__FILE__)."/actions/$actionname.action.php";
        $actionclass = $actionname.'Action';
        if (file_exists($actionfile)) include($actionfile);
        else return trigger_error("Action file '$actionfile' not found.", E_USER_ERROR);
        if (class_exists($actionclass))  $action = new $actionclass();
        else return trigger_error("Action class '$actionclass' not found.", E_USER_ERROR);
        $action->execute($this, $this->request);
    }


    function view($viewname)
    {
        $viewfile = dirname(__FILE__)."/views/$viewname.view.php";
        $viewclass = $viewname.'View';
        if (file_exists($viewfile)) include($viewfile);
        #else return trigger_error("View file '$viewfile' not found.", E_USER_ERROR);
        if (class_exists($viewclass))  $view = new $viewclass();
        else return trigger_error("View class '$viewclass' not found.", E_USER_ERROR);
        $view->render($this->renderer, $this->request);
    }
    function redirect($action=null)
    {
        $url  = $_SERVER['PHP_SELF'].'?option='.$this->name;
        $url .= !is_null($action) ? '&task='.$action : '';
        if (headers_sent()) {
            echo "<script>document.location.href='$url';</script>";
        } else {
            #if (ob_get_contents()) while (@ob_end_clean()); // clear output buffer if one exists
            header( "Location: $url" );
        }
        exit;
    }
}

class Action
{
    var $view;

    function Action()
    {
    }
    function execute(&$controller, &$request)
    {
        return trigger_error('Action::execute() must be overridden.', E_USER_ERROR);
    }
    function setView($view)
    {
        $this->viewname = $view;
    }
    function getView()
    {
        return $this->view;
    }
}

class View
{

    function View(){}
    function render(&$request, &$renderer)
    {
        return trigger_error('View::render() must be overridden');
    }
}

class Renderer
{

    var $dir;
    var $vars = array();
    var $engine = 'php';
    var $template = '';
    var $debug = 0;

    function Renderer(){}

    function &getInstance($type = 'php') {
        static $renderer;
        if (is_null($renderer[$type])) {
            if ($type == 'php') {
                $renderer[$type] = new Renderer();
            } else {
                $classname = $type . 'Renderer';
                if (class_exists($classname))
                $renderer[$type] = new $classname();
            }
        }
        return $renderer[$type];
    }

    function display($template, $return = false){
        if ($template == NULL){
            return trigger_error('A template has not been specified', E_USER_ERROR);
        }
        $this->template = $this->dir . $template;
        if ($this->debug) echo nl2br($this->template."\n");

        if (is_readable($this->template)) {
            extract($this->getvars());
            if ($return) {
                ob_start();
                include_once($this->template);
                $ret = ob_get_contents();
                ob_end_clean();
                return $ret;
            } else {
                include_once($this->template);
            }
        } else {
            return trigger_error("Template file $template does not exist or is not readable", E_USER_ERROR);
        }
        return false;
    }

    function fetch($template){
        return $this->display($template, true);
    }

    function &getengine(){
        return $this->engine;
    }

    function addvar($key, $value){
        $this->vars[$key] = $value;
    }

    function addbyref ($key, &$value) {
        $this->vars[$key] = $value;
    }

    function getvars($name = false){
        return (isset($this->vars[$name])) ? $this->vars[$name] : $this->vars;
    }

    function setdir($dir){
        $this->dir = (substr($dir, -1) == DIRECTORY_SEPARATOR) ? $dir : $dir.DIRECTORY_SEPARATOR;
    }

    function getdir(){
        return $this->dir;
    }

    function settemplate($template){
        $this->template = $template;
    }
}
/*
class ToolBar
{

var $map 	 = array();
var $name 	 = null;
var $request = null;

function ToolBar($name)
{
$this->name = $name;
$this->request = $_REQUEST;
}
function display($method = null)
{
$request = isset($this->request['task']) ? $this->request['task'] : '_default';
if (is_null($method))
$method = $request;
if (in_array($method,  array_keys($this->map)))
$method = $this->map[$method];
if ($method && strlen($method) > 2 && method_exists($this, $method))
return $this->$method();
return false;
}
function _default(){}
}
*/
class XMLUtils
{
    function parse_into_array($xml) {
        $p = xml_parser_create();
        xml_parser_set_option($p, XML_OPTION_CASE_FOLDING, 0);
        xml_parser_set_option($p, XML_OPTION_SKIP_WHITE, 1);
        xml_parse_into_struct($p, $xml, $values);
        xml_parser_free($p);
        $current = $prev = array();
        $xmlarray =& $current;
        foreach($values as $key => $value) {
            $index = count($xmlarray);
            switch ($value['type'])
            {
                case 'open':
                $xmlarray[$index] = array();
                $xmlarray[$index]['tag']        = isset($value["tag"]) ? $value["tag"] : null;
                $xmlarray[$index]['value']      = isset($value["value"]) ? $value["value"] : null;
                $xmlarray[$index]['attributes'] = isset($value["attributes"]) ? $value["attributes"] : null;
                $xmlarray[$index]['nodes']   = array();
                $prev[count($prev)] = &$xmlarray;
                $xmlarray = &$xmlarray[$index]['nodes'];
                break;
                case 'complete':
                $xmlarray[$index] = array();
                $xmlarray[$index]['tag']        = isset($value["tag"]) ? $value["tag"] : null;
                $xmlarray[$index]['value']      = isset($value["value"]) ? $value["value"] : null;
                $xmlarray[$index]['attributes'] = isset($value["attributes"]) ? $value["attributes"] : null;
                break;
                case 'close':
                $xmlarray = &$prev[count($prev) - 1];
                unset($prev[count($prev) - 1]);
                break;
            }
        }
        return $xmlarray;
    }
    function parse_file_into_array($file) {
        return XMLUtils::parse_into_array(file_get_contents($file));
    }

    function array_to_xml($array, $encoding='utf-8') {
        $xml = "<?xml version=\"1.0\" encoding=\"$encoding\"?>\n";
        if ((!empty($array)) AND (is_array($array))) {
            foreach ($array as $key => $value) {
                switch ($value["type"]) {
                    case "open":
                    $xml .= str_repeat("\t", $value["level"] - 1);
                    $xml .= "<" . strtolower($value["tag"]);
                    if (isset($value["attributes"])) {
                        foreach ($value["attributes"] as $k => $v) {
                            $xml .= sprintf(' %s="%s"', strtolower($k), $v);
                        }
                    }
                    $xml .= ">\n";
                    break;
                    case "complete":
                    $xml .= str_repeat("\t", $value["level"] - 1);
                    $xml .= "<" . strtolower($value["tag"]);
                    if (isset($value["attributes"])) {
                        foreach ($value["attributes"] as $k => $v) {
                            $xml .= sprintf(' %s="%s"', strtolower($k), $v);
                        }
                    }
                    $xml .= ">";
                    $xml .= isset($value['value']) ? $value['value'] : false;
                    $xml .= "</".strtolower($value["tag"]).">\n";
                    break;
                    case "close":
                    $xml .= str_repeat("\t", $value["level"] - 1);
                    $xml .= "</" . strtolower($value["tag"]) . ">\n";
                    break;
                    default:
                    break;
                }
            }
        }
        return $xml;
    }
}


class XmlWriter {

    var $xml;
    var $indent;
    var $stack = array();

    function XmlWriter($encoding='UTF-8', $indent='  ') {
        $this->indent = $indent;
        $this->xml = '<?xml version="1.0" encoding="'.$encoding.'"?>'."\n";
    }
    function _indent() {
        for ($i = 0, $j = count($this->stack); $i < $j; $i++) {
            $this->xml .= $this->indent;
        }
    }
    function open($element, $attributes = array()) {
        $this->_indent();
        $this->xml .= '<'.$element;
        foreach ($attributes as $key => $value) {
            $this->xml .= ' '.$key.'="'.htmlentities($value).'"';
        }
        $this->xml .= ">\n";
        $this->stack[] = $element;
    }
    function element($element, $content, $attributes = array()) {
        $this->_indent();
        $this->xml .= '<'.$element;
        foreach ($attributes as $key => $value) {
            $this->xml .= ' '.$key.'="'.htmlentities($value).'"';
        }
        if (!preg_match("/\<\!\[CDATA\[(.*)\]\]\>/", $content)) {
            $content = htmlentities($content);
        }
        $this->xml .= '>'.$content.'</'.$element.'>'."\n";
    }
    function emptyelement($element, $attributes = array()) {
        $this->_indent();
        $this->xml .= '<'.$element;
        foreach ($attributes as $key => $value) {
            $this->xml .= ' '.$key.'="'.htmlentities($value).'"';
        }
        $this->xml .= " />\n";
    }
    function close() {
        $element = array_pop($this->stack);
        $this->_indent();
        $this->xml .= "</$element>\n";
    }
    function getXml() {
        return $this->xml;
    }
}

?>